package com.nisum.app.controller;

import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.Duration;

@RestController
public class FluxAndMonoController {

    //This will return all data at a time not event by event
    // (internally following events but not visible on postman)
    @GetMapping("/flux")
    public Flux<Integer> getFlux(){

        return Flux.just(1,2,3,4)
                .log();
    }


    //In this case every event is emitted after one sec
    // but in postman it gives all data after 4 sec becoz it assume it is JSON response
    /*@GetMapping("/flux")
    public Flux<Integer> getFlux(){

        return Flux.just(1,2,3,4)
                .delayElements(Duration.ofSeconds(1))
                .log();
    }*/

   //we can get events after 1 sec
    /*@GetMapping(value = "/fluxstream"
            ,produces = MediaType.APPLICATION_NDJSON_VALUE)
    public Flux<Integer> getFluxStream(){

        return Flux.just(1,2,3,4)
                .delayElements(Duration.ofSeconds(1))
                .log();
    }*/

    @GetMapping(value = "/fluxstream"
            ,produces = MediaType.APPLICATION_STREAM_JSON_VALUE)
    public Flux<Long> getFluxStream(){

        return Flux.interval(Duration.ofSeconds(1))
                .log();
    }

    @GetMapping("/mono")
    public Mono<Integer> getMono(){

        return Mono.just(1)
                .log();
    }
}
